#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "m_data.h"
#include "m_sr.h"

/* Save, restore routines */

void SaveMusStatus(struct MusData *Buf)
{
	int p = MusData.Paused;
	SetPause(1);
    FreeUncompressedData();
    memcpy(Buf, &MusData, sizeof(*Buf));
    /* FIXME: Save music device status */
    SetPause(p);
}

void RestoreMusStatus(const struct MusData *Buf)
{
	SetPause(1);
    memcpy(&MusData, Buf, sizeof(*Buf));
    if(MusData.driver)MusData.driver->reset();
    UncompressChannels();
    /* Paused flag was saved as 1 */
    SetPause(0);
}

void FreeUncompressedData(void)
{
	unsigned a;
	for(a=0; a<MAXCHN; ++a)
	{
		if(MusData.PlayData.ch[a].patndata)
			free(MusData.PlayData.ch[a].patndata);
	}
}

void UncompressChannels(void)
{
	unsigned a;
	
	const struct InternalChanData *chans = MusData.SongData.Chans[MusData.PlayData.posi.Pat];
	
	/*printf("Pattern %u:\n", MusData.PlayData.posi.Pat);*/
	
	for(a=0; a<MusData.SongData.Hdr->ChanCount; ++a)
	{
		const BYTE *src = (const BYTE *)chans + sizeof(*chans);
		BYTE *dest = (BYTE *)calloc(chans->OriginalByteSize, 1); /* We want zero */
		BYTE bbb, ooo, ppp, limit;
		unsigned srcbitpos=0, destbitpos=0;
		
		if(!dest)
		{
			fprintf(stderr, "h? got NULL, originalbytesize=%u\n", chans->OriginalByteSize);
		}
		
		MusData.PlayData.ch[a].patndata = dest;
		
		bbb = ReadBits(src, srcbitpos, 4); srcbitpos += 4;
		ooo = ReadBits(src, srcbitpos, 4); srcbitpos += 4;
		ppp = ReadBits(src, srcbitpos, 2); srcbitpos += 2;
		
		limit = 1 + bbb + ooo;
		
		/*printf("  a=%u: bbb=%u, ooo=%u, ppp=%u\n", a, bbb, ooo, ppp);*/
		
		/* DWORD ReadBits(const BYTE *data, unsigned bitpos, unsigned bitcount)       */
		/* void WriteBits(BYTE *dest, unsigned bitpos, unsigned bitcount, DWORD data) */

		for(; srcbitpos < chans->BitCount; )
		{
			unsigned b = ReadBits(src, srcbitpos, 2);
			if(b == ppp)
			{
				srcbitpos += 2;
				if(ReadBits(src, srcbitpos, 1))
				{
    				WORD bb = ReadBits(src, srcbitpos+1,     bbb);
    				WORD oo = ReadBits(src, srcbitpos+1+bbb, ooo);
    				srcbitpos += limit; /* 1+bbb+ooo */
    				
    				/*printf("     Got bb=%u, oo=%u (destbitpos=%u).\n", bb,oo, destbitpos);*/
    				CopyBits(dest, destbitpos, dest, destbitpos-oo-1, bb+limit+3);

    				destbitpos += bb+limit+3;
				}
				else
				{
					/*printf("     Got ppp=%u at destbitpos=%u\n", ppp, destbitpos);*/
					WriteBits(dest, destbitpos, 2, ppp);
					++srcbitpos;
					destbitpos += 2;
				}
				continue;
			}
			WriteBits(dest, destbitpos++, 1, ReadBits(src, srcbitpos++, 1));
		}
		
		chans = (const struct InternalChanData *)((const BYTE *)chans + ((chans->BitCount+7) >> 3) + sizeof(*chans));
	}
}
